Mask

Struct Mask 

pub struct Mask<T, const N: usize>(/* private fields */)
where
    T: MaskElement;
๐Ÿ”ฌThis is a nightly-only experimental API. (portable_simd #86656)
A SIMD vector mask for N elements of width specified by Element.

Masks represent boolean inclusion/exclusion on a per-element basis.

The layout of this type is unspecified, and may change between platforms and/or Rust versions, and code should not assume that it is equivalent to [T; N].

N cannot be 0 and may be at most 64. This limit may be increased in the future.

Implementations

Source

impl<T, const N: usize> Mask<T, N>
where T: MaskElement,

pub fn reverse(self) -> Self

๐Ÿ”ฌThis is a nightly-only experimental API. (portable_simd #86656)

Reverse the order of the elements in the mask.

pub fn rotate_elements_left<const OFFSET: usize>(self) -> Self

๐Ÿ”ฌThis is a nightly-only experimental API. (portable_simd #86656)

Rotates the mask such that the first OFFSET elements of the slice move to the end while the last self.len() - OFFSET elements move to the front. After calling rotate_elements_left, the element previously at index OFFSET will become the first element in the slice.

pub fn rotate_elements_right<const OFFSET: usize>(self) -> Self

๐Ÿ”ฌThis is a nightly-only experimental API. (portable_simd #86656)

Rotates the mask such that the first self.len() - OFFSET elements of the mask move to the end while the last OFFSET elements move to the front. After calling rotate_elements_right, the element previously at index self.len() - OFFSET will become the first element in the slice.

pub fn shift_elements_left<const OFFSET: usize>(self, padding: bool) -> Self

๐Ÿ”ฌThis is a nightly-only experimental API. (portable_simd #86656)

Shifts the mask elements to the left by OFFSET, filling in with padding from the right.

pub fn shift_elements_right<const OFFSET: usize>(self, padding: bool) -> Self

๐Ÿ”ฌThis is a nightly-only experimental API. (portable_simd #86656)

Shifts the mask elements to the right by OFFSET, filling in with padding from the left.

pub fn interleave(self, other: Self) -> (Self, Self)

๐Ÿ”ฌThis is a nightly-only experimental API. (portable_simd #86656)

Interleave two masks.

The resulting masks contain elements taken alternatively from self and other, first filling the first result, and then the second.

The reverse of this operation is Mask::deinterleave.

let a = mask32x4::from_array([false, true, false, true]);
let b = mask32x4::from_array([false, false, true, true]);
let (x, y) = a.interleave(b);
assert_eq!(x.to_array(), [false, false, true, false]);
assert_eq!(y.to_array(), [false, true, true, true]);
pub fn deinterleave(self, other: Self) -> (Self, Self)

๐Ÿ”ฌThis is a nightly-only experimental API. (portable_simd #86656)

Deinterleave two masks.

The first result takes every other element of self and then other, starting with the first element.

The second result takes every other element of self and then other, starting with the second element.

The reverse of this operation is Mask::interleave.

let a = mask32x4::from_array([false, true, false, true]);
let b = mask32x4::from_array([false, false, true, true]);
let (x, y) = a.deinterleave(b);
assert_eq!(x.to_array(), [false, false, false, true]);
assert_eq!(y.to_array(), [true, true, false, true]);
pub fn resize<const M: usize>(self, value: bool) -> Mask<T, M>

๐Ÿ”ฌThis is a nightly-only experimental API. (portable_simd #86656)

Resize a mask.

If M > N, extends the length of a mask, setting the new elements to value. If M < N, truncates the mask to the first M elements.

let x = mask32x4::from_array([false, true, true, false]);
assert_eq!(x.resize::<8>(true).to_array(), [false, true, true, false, true, true, true, true]);
assert_eq!(x.resize::<2>(true).to_array(), [false, true]);
pub fn extract<const START: usize, const LEN: usize>(self) -> Mask<T, LEN>

๐Ÿ”ฌThis is a nightly-only experimental API. (portable_simd #86656)

Extract a vector from another vector.

let x = mask32x4::from_array([false, true, true, false]);
assert_eq!(x.extract::<1, 2>().to_array(), [true, true]);
Source

impl<T, const N: usize> Mask<T, N>
where T: MaskElement,

pub const fn splat(value: bool) -> Self

๐Ÿ”ฌThis is a nightly-only experimental API. (portable_simd #86656)

Constructs a mask by setting all elements to the given value.

pub fn from_array(array: [bool; N]) -> Self

๐Ÿ”ฌThis is a nightly-only experimental API. (portable_simd #86656)

Converts an array of bools to a SIMD mask.

pub fn to_array(self) -> [bool; N]

๐Ÿ”ฌThis is a nightly-only experimental API. (portable_simd #86656)

Converts a SIMD mask to an array of bools.

pub unsafe fn from_simd_unchecked(value: Simd<T, N>) -> Self

๐Ÿ”ฌThis is a nightly-only experimental API. (portable_simd #86656)

Converts a vector of integers to a mask, where 0 represents false and -1 represents true.

Safety

All elements must be either 0 or -1.

pub fn from_simd(value: Simd<T, N>) -> Self

๐Ÿ”ฌThis is a nightly-only experimental API. (portable_simd #86656)

Converts a vector of integers to a mask, where 0 represents false and -1 represents true.

Panics

Panics if any element is not 0 or -1.

pub fn to_simd(self) -> Simd<T, N>

๐Ÿ”ฌThis is a nightly-only experimental API. (portable_simd #86656)

Converts the mask to a vector of integers, where 0 represents false and -1 represents true.

pub fn cast<U: MaskElement>(self) -> Mask<U, N>

๐Ÿ”ฌThis is a nightly-only experimental API. (portable_simd #86656)

Converts the mask to a mask of any other element size.

pub unsafe fn test_unchecked(&self, index: usize) -> bool

๐Ÿ”ฌThis is a nightly-only experimental API. (portable_simd #86656)

Tests the value of the specified element.

Safety

index must be less than self.len().

pub fn test(&self, index: usize) -> bool

๐Ÿ”ฌThis is a nightly-only experimental API. (portable_simd #86656)

Tests the value of the specified element.

Panics

Panics if index is greater than or equal to the number of elements in the vector.

pub unsafe fn set_unchecked(&mut self, index: usize, value: bool)

๐Ÿ”ฌThis is a nightly-only experimental API. (portable_simd #86656)

Sets the value of the specified element.

Safety

index must be less than self.len().

pub fn set(&mut self, index: usize, value: bool)

๐Ÿ”ฌThis is a nightly-only experimental API. (portable_simd #86656)

Sets the value of the specified element.

Panics

Panics if index is greater than or equal to the number of elements in the vector.

pub fn any(self) -> bool

๐Ÿ”ฌThis is a nightly-only experimental API. (portable_simd #86656)

Returns true if any element is set, or false otherwise.

pub fn all(self) -> bool

๐Ÿ”ฌThis is a nightly-only experimental API. (portable_simd #86656)

Returns true if all elements are set, or false otherwise.

pub fn to_bitmask(self) -> u64

๐Ÿ”ฌThis is a nightly-only experimental API. (portable_simd #86656)

Creates a bitmask from a mask.

Each bit is set if the corresponding element in the mask is true.

pub fn from_bitmask(bitmask: u64) -> Self

๐Ÿ”ฌThis is a nightly-only experimental API. (portable_simd #86656)

Creates a mask from a bitmask.

For each bit, if it is set, the corresponding element in the mask is set to true. If the mask contains more than 64 elements, the remainder are set to false.

pub fn first_set(self) -> Option<usize>

๐Ÿ”ฌThis is a nightly-only experimental API. (portable_simd #86656)

Finds the index of the first set element.

assert_eq!(mask32x8::splat(false).first_set(), None);
assert_eq!(mask32x8::splat(true).first_set(), Some(0));

let mask = mask32x8::from_array([false, true, false, false, true, false, false, true]);
assert_eq!(mask.first_set(), Some(1));